package ru.dbotthepony.kstarbound.defs.animation

import ru.dbotthepony.kstarbound.defs.AssetReference
import ru.dbotthepony.kstarbound.defs.image.SpriteReference
import ru.dbotthepony.kstarbound.io.json.builder.JsonFactory
import ru.dbotthepony.kvector.vector.ndouble.Vector2d
import ru.dbotthepony.kvector.vector.ndouble.Vector4d

@JsonFactory
data class ParticleDefinition(
	val count: Int = 1,
	val offset: Vector2d? = null,
	val offsetRegion: Vector4d? = null,
	val particle: Config,
) {
	@JsonFactory
	data class Config(
		val type: ParticleType,
		val animation: AssetReference<AnimationDefinition>? = null,
		val image: SpriteReference? = null,
		val position: Vector2d = Vector2d.ZERO,
		val offsetRegion: Vector4d = Vector4d.ZERO,
		val initialVelocity: Vector2d = Vector2d.ZERO,
		val finalVelocity: Vector2d = Vector2d.ZERO,
		val approach: Vector2d = Vector2d.ZERO,
		val angularVelocity: Double = 0.0,
		val destructionAction: DestructionAction? = null,
		val destructionTime: Double? = null,
		val fade: Double = 0.0,
		val size: Double = 1.0,
		val layer: ParticleLayer? = null,
		val timeToLive: Double = 1.0,
		val variance: Variance = Variance.EMPTY
	)

	@JsonFactory
	data class Variance(
		val initialVelocity: Vector2d = Vector2d.ZERO,
		val position: Vector2d = Vector2d.ZERO,
		val angularVelocity: Double = 0.0
	) {
		companion object {
			val EMPTY = Variance()
		}
	}
}
